function [eiggap1,eiggap2,eiggap3,conH] =CondBestNonSym3(A,X)
% Compute an estimate of the condition number of the best approximation
% problem, see (5.17)  and || K^{-1} || in E&S (2011)
% A is (1,2)-symmetric

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Distributed under a 3-Clause BSD License 
%
% Copyright 2021 Lars Eldén, Linköping University, and Maryam Dehghan,
% Federal University of Ceará,
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% One step of BKS  with stage = 1, p irrelevant 
r1 = size(X{1},2);
r2 = size(X{2},2);
r3 = size(X{3},2); 
[C,U1,V1,W1]=BKNonSym(A,X{1},X{2},X{3},[],1);
F=C(1:r1,1:r1,1:r3);
FF1=double(ttt(F,F,[2 3],[2 3]));
eigF1=sort(sqrt(eig(FF1))','descend');
Fp1=C(r1+1:end,1:r1,1:r3);
FFp1=double(ttt(Fp1,Fp1,[2 3],[2 3]));
lpmax=sqrt(max(eig(FFp1)));
eiggap1=[eigF1 lpmax]; 

FF2=double(ttt(F,F,[1 3],[1 3]));
eigF2=sort(sqrt(eig(FF2))','descend');
Fp2=C(1:r1,r2+1:end,1:r3);
FFp2=double(ttt(Fp2,Fp2,[1 3],[1 3]));
lpmax=sqrt(max(eig(FFp2)));
eiggap2=[eigF2 lpmax];

FF3=double(ttt(F,F,[1 2],[1 2]));
eigF3=sort(sqrt(eig(FF3))','descend');
Fp3=C(1:r1,1:r1,r3+1:end);
FFp3=double(ttt(Fp3,Fp3,[1 2],[1 2]));
lpmax=sqrt(max(eig(FFp3)));
eiggap3=[eigF3 lpmax];

r=[r1 r1 r3];
n=size(C);
E1=[eye(r1);zeros(n(1)-r1,r1)];
E2=[eye(r2);zeros(n(2)-r2,r2)];
E3=[eye(r3);zeros(n(3)-r3,r3)];
H=Hessian(C,{E1,E1,E3},n,r);
eigH=eig(H);
conH=max(eigH)/min(eigH);

end

