function [X, f, nn, itnn] = algALS(A,X,k,maxIt,tol,itstep)
% Alternating Least Squares algorithm for tensor approximation.
% Returns the end-points in X, the norm of the gradient
% and the values of the objektive function.
% itstep: compute the gradient every itstep iterations
% nn:     the relative gradient
% itnn:   the step numbers where the gradient was computed
% Copyright 2008.
% Berkant Savas, Link�ping University.
% LE 2021

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Distributed under a 3-Clause BSD License 
%
% Copyright 2021 Lars Eldén, Linköping University, and Maryam Dehghan,
% Federal University of Ceará,
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin==5
    itstep=1;
end
it = 1; itnn=[]; nn=[];
done = 0;
while ~done && it <= maxIt
    % Update the three matrices
    for i = 1:ndims(A)
        An = ttm(A, X, -i,'t');
        [Un,s,v] = svd(tam(An, i),'econ');
        X{i} = Un(:,1:k(i));
    end
    
    % Compute the function value and norm of the gradient.
    T = ttm(A,X,'t');
    f(it,1) = norm(T)^2/2;
    
    %     for i = 1:ndims(A)          % Very memory inefficient
    %         P{i} = eye(size(X{i},1)) - X{i}*X{i}';
    %     end
    if it==1 | round(it/itstep)*itstep==it    % compute the gradient every itstep steps
        %     if ndims(A) == 3
        %         T1 = ttm(A ,X{3},3,'t');
        %         T2 = ttm(T1,X{1},1,'t');
        %         T1 = ttm(T1,X{2},2,'t');
        %         T3 = ttm(A,{X{1},X{2}},[1,2],'t');
        %         F  = ttm(T2,X{2},2,'t');
        %
        %         g{1} = -tam(T1,1)*tam(F,1)';
        %         g{2} = -tam(T2,2)*tam(F,2)';
        %         g{3} = -tam(T3,3)*tam(F,3)';
        %         g{1} = P{1}*g{1};
        %         g{2} = P{2}*g{2};
        %         g{3} = P{3}*g{3};
        %     else
        F  = ttm(A,X,'t');       % This is much more memory and execution efficient than the
        g = cell(1,ndims(A));    % above code
        for i = 1:ndims(A)
            g{i} = - tam(ttm(A,X,-i,'t'),i) *tam(F,i)';
            g{i} = g{i}-X{i}*(X{i}'*g{i});
        end
        
        %    end
        nnit = 0;
        for i = 1:ndims(A)
            nnit=nnit + norm(g{i},'fro');
        end
        nn=[nn nnit];
        if nn(it)/f(it) < tol
            done   = 1;
        end
        itnn=[itnn it];
    end
    it=it+1;
end
 
