function [P1]=scaleMatrixNonSym(P)
% P is a nonsymmetric  matrix
% Scale P1=D_1^(-1/2)*P*D_2^(-1/2), where D_i=diag(d_i), d_1=e'*P, and d_2=P*e.
% e is a vector of ones

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Distributed under a 3-Clause BSD License 
%
% Copyright 2021 Lars Eldén, Linköping University, and Maryam Dehghan,
% Federal University of Ceará,
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[m1,m2]=size(P);
d1=ones(1,m1)*P;
w1=1./sqrt(d1);
ii=find(w1==inf);
w1(ii)=0;
W2=spdiags(w1',0,m2,m2);


d2=P*ones(m2,1);
w2=1./sqrt(d2);
ii=find(w2==inf);
w2(ii)=0;
W1=spdiags(w2,0,m1,m1);
P1=W1*P*W2;

end
